/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game; 

var nWidth = 640;
var nHeight = 480;


// les variables du jeu
var hexagonWidth = 70;
var hexagonHeight = 80;
var gridSizeX = 17;
var gridSizeY = 7;
var columns = [Math.ceil(gridSizeX/2),Math.floor(gridSizeX/2)];
var moveIndex;
var sectorWidth = hexagonWidth;
var sectorHeight = hexagonHeight/4*3;
var gradient = (hexagonHeight/4)/(hexagonWidth/2);
var marker;
var hexagonGroup;
var hexagonArray = [];



document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API canvas
    game = new Phaser.Game(nWidth, nHeight, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});

    // les variables globales du jeu
    game.global = {
        thumbRows : 5// nombre d'icone par ligne  game.global.thumbRows
    }

}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    // charge les animations / graphismes du jeu
    game.load.image("hexagon", "assets/hexagon.png");
    game.load.image("marker", "assets/marker.png");
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du jeu
}


/**
 * Affiche le jeu en plein écran
 * 
 * @return {[type]} [description]
 */
function goFullScreen()
{
    game.scale.pageAlignHorizontally = true;
    game.scale.pageAlignVertically = true;
    game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
    game.scale.setScreenSize(true);
}


/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    hexagonGroup = game.add.group();

    // construction de la grille du démineur
    for(var i = 0; i < gridSizeY/2; i ++)
    {
        hexagonArray[i] = [];

        for(var j = 0; j < gridSizeX; j ++)
        {
            if(gridSizeY%2==0 || i+1<gridSizeY/2 || j%2==0)
            {
                // les coordonnées de la forme
                var hexagonX = hexagonWidth*j/2;
                var hexagonY = hexagonHeight*i*1.5+(hexagonHeight/4*3)*(j%2); 

                // ajoute la forme
                var hexagon = game.add.sprite(hexagonX,hexagonY,"hexagon");
                hexagonGroup.add(hexagon);
                hexagonArray[i][j]=hexagon;

                // le séparateur des coordonnées de la forme
                var hexagonText = game.add.text(hexagonX+hexagonWidth/3.2,hexagonY+15,i+"; "+j);
                hexagonText.font = "arial";
                hexagonText.fontSize = 15;
                hexagonGroup.add(hexagonText);
            }
        }
    }

    // positionne le groupe forme + texte
    hexagonGroup.x = (game.width-hexagonWidth*Math.ceil(gridSizeX/2))/2;
    
    if(gridSizeX%2==0)
        hexagonGroup.x-=hexagonWidth/4;
    
    // positionne le groupe forme + texte  
    hexagonGroup.y = (game.height-Math.ceil(gridSizeY/2)*hexagonHeight-Math.floor(gridSizeY/2)*hexagonHeight/2)/2;
    
    if(gridSizeY%2==0)
        hexagonGroup.y-=hexagonHeight/8;
    
    // création du markeur du démineur  
    marker = game.add.sprite(0,0,"marker");
    marker.anchor.setTo(0.5);
    marker.visible=false;// masqué à l'utilisateur

    hexagonGroup.add(marker);

    // écouteur d'événement - appel checkForme
    moveIndex = game.input.addMoveCallback(checkForme, this);       
}


/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
}


/**
 * Vérifie la forme pointée
 * lors du déplacement de la souris
 * 
 * @return {[type]} [description]
 */
function checkForme()
{
    // détermine les coordonnées en x et en y
    var candidateX = Math.floor((game.input.worldX-hexagonGroup.x)/sectorWidth);
    var candidateY = Math.floor((game.input.worldY-hexagonGroup.y)/sectorHeight);

    var deltaX = (game.input.worldX-hexagonGroup.x)%sectorWidth;
    var deltaY = (game.input.worldY-hexagonGroup.y)%sectorHeight; 

    if(candidateY%2==0)
    {
        if(deltaY<((hexagonHeight/4)-deltaX*gradient))
        {
            candidateX--;
            candidateY--;
        }

        if(deltaY<((-hexagonHeight/4)+deltaX*gradient))
            candidateY--;

    }    
    else
    {
       if(deltaX>=hexagonWidth/2)
       {
            if( deltaY < (hexagonHeight/2-deltaX*gradient) )
                candidateY--;
       }
       else
       {
            if( deltaY < deltaX*gradient )
                candidateY--;
            else
                candidateX--;
       }
    }

    updateForme(candidateX,candidateY);  
}
     

/**
 * Mets à jour la couleur des formes 
 * en fonction de la position du pointeur de la souris
 * 
 * @param  {[type]} posX [description]
 * @param  {[type]} posY [description]
 * @return {[type]}      [description]
 */
function updateForme(posX,posY)
{
    for(var i = 0; i < gridSizeY/2; i ++)
    {
        for(var j = 0; j < gridSizeX; j ++)
        {
            if(gridSizeY%2==0 || i+1<gridSizeY/2 || j%2==0)
                hexagonArray[i][j].tint = 0xffffff;
        }
    }

    if(posX<0 || posY<0 || posY>=gridSizeY || posX>columns[posY%2]-1)
        marker.visible=false;// cache le pointeur si il n'est pas au dessus d'une forme ou en dehors de l'écran
    else
    {
        marker.visible=true;
        marker.x = hexagonWidth*posX;
        marker.y = hexagonHeight/4*3*posY+hexagonHeight/2;
        
        if(posY%2==0)
            marker.x += hexagonWidth/2;
        else
            marker.x += hexagonWidth;
        
        var markerX = posX*2+posY%2;
        var markerY = Math.floor(posY/2);

        // modifie la couleur de la forme sous la souris
        hexagonArray[markerY][markerX].tint = 0xFFFF00;


        // -- modifie la couleur des formes adjacentes --
        
        // à gauche
        if(markerX-2>=0)
            hexagonArray[markerY][markerX-2].tint = 0x00C5DF;
    
        // à droite
        if(markerX+2<gridSizeX)
            hexagonArray[markerY][markerX+2].tint = 0x00C5DF;
        
        // au dessus
        if(markerY-1+markerX%2>=0)
        {
            // à gauche
            if(markerX-1>=0)
                hexagonArray[markerY-1+markerX%2][markerX-1].tint = 0x00C5DF;
            
            // à droite
            if(markerX+1<gridSizeX)
                hexagonArray[markerY-1+markerX%2][markerX+1].tint = 0x00C5DF;
        }

        // en dessous
        if(markerY+markerX%2<gridSizeY/2 && (gridSizeY%2==0 || markerY<Math.floor(gridSizeY/2)))
        {
            // à gauche
            if(markerX-1>=0)
                hexagonArray[markerY+markerX%2][markerX-1].tint = 0x00C5DF;
            
            // à droite
            if(markerX+1<gridSizeX)
                hexagonArray[markerY+markerX%2][markerX+1].tint = 0x00C5DF;
        } 
    }
}   



